/*
 * Copyright 2015, Yahoo Inc.
 * Copyrights licensed under the New BSD License.
 * See the accompanying LICENSE file for terms.
 */

'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _react = require('react');

var _intlMessageformat = require('intl-messageformat');

var _intlMessageformat2 = _interopRequireDefault(_intlMessageformat);

var _intlRelativeformat = require('intl-relativeformat');

var _intlRelativeformat2 = _interopRequireDefault(_intlRelativeformat);

var _plural = require('../plural');

var _plural2 = _interopRequireDefault(_plural);

var _intlFormatCache = require('intl-format-cache');

var _intlFormatCache2 = _interopRequireDefault(_intlFormatCache);

var _utils = require('../utils');

var _types = require('../types');

var _format = require('../format');

var format = _interopRequireWildcard(_format);

var _localeDataRegistry = require('../locale-data-registry');

var intlPropNames = Object.keys(_types.intlPropTypes);
var intlFormatPropNames = Object.keys(_types.intlFormatPropTypes);

var IntlProvider = (function (_Component) {
    _inherits(IntlProvider, _Component);

    function IntlProvider(props) {
        var _this = this;

        _classCallCheck(this, IntlProvider);

        _Component.call(this, props);

        // Used to stabilize time when performing an initial rendering so that
        // all relative times use the same reference "now" time.
        var initialNow = isFinite(props.initialNow) ? Number(props.initialNow) : Date.now();

        this.state = {
            // Creating `Intl*` formatters is expensive so these format caches
            // memoize the `Intl*` constructors and have the same lifecycle as
            // this IntlProvider instance.
            getDateTimeFormat: _intlFormatCache2['default'](Intl.DateTimeFormat),
            getNumberFormat: _intlFormatCache2['default'](Intl.NumberFormat),
            getMessageFormat: _intlFormatCache2['default'](_intlMessageformat2['default']),
            getRelativeFormat: _intlFormatCache2['default'](_intlRelativeformat2['default']),
            getPluralFormat: _intlFormatCache2['default'](_plural2['default']),

            // Wrapper to provide stable "now" time for initial render.
            now: function now() {
                return _this._didDisplay ? Date.now() : initialNow;
            }
        };
    }

    IntlProvider.prototype.getConfig = function getConfig() {
        var _this2 = this;

        var config = intlPropNames.reduce(function (config, name) {
            config[name] = _this2.props[name];
            return config;
        }, {});

        if (!_localeDataRegistry.hasLocaleData(config.locale)) {
            var _config = config;
            var locale = _config.locale;
            var defaultLocale = _config.defaultLocale;
            var defaultFormats = _config.defaultFormats;

            if (process.env.NODE_ENV !== 'production') {
                console.error('[React Intl] Missing locale data for: "' + locale + '". ' + ('Using default locale: "' + defaultLocale + '" as fallback.'));
            }

            // Since there's no registered locale data for `locale`, this will
            // fallback to the `defaultLocale` to make sure things can render.
            // The `messages` are overridden to the `defaultProps` empty object
            // to maintain referential equality across re-renders. It's assumed
            // each <FormattedMessage> contains a `defaultMessage` prop.
            config = _extends({}, config, {
                locale: defaultLocale,
                formats: defaultFormats,
                messages: IntlProvider.defaultProps.messages
            });
        }

        return config;
    };

    IntlProvider.prototype.getBoundFormatFns = function getBoundFormatFns(config, state) {
        return intlFormatPropNames.reduce(function (boundFormatFns, name) {
            boundFormatFns[name] = format[name].bind(null, config, state);
            return boundFormatFns;
        }, {});
    };

    IntlProvider.prototype.getChildContext = function getChildContext() {
        var config = this.getConfig();

        // Bind intl factories and current config to the format functions.
        var boundFormatFns = this.getBoundFormatFns(config, this.state);

        return {
            intl: _extends({}, config, boundFormatFns, {
                now: this.state.now
            })
        };
    };

    IntlProvider.prototype.shouldComponentUpdate = function shouldComponentUpdate() {
        for (var _len = arguments.length, next = Array(_len), _key = 0; _key < _len; _key++) {
            next[_key] = arguments[_key];
        }

        return _utils.shouldIntlComponentUpdate.apply(undefined, [this].concat(next));
    };

    IntlProvider.prototype.componentDidMount = function componentDidMount() {
        this._didDisplay = true;
    };

    IntlProvider.prototype.render = function render() {
        return _react.Children.only(this.props.children);
    };

    return IntlProvider;
})(_react.Component);

exports['default'] = IntlProvider;

IntlProvider.displayName = 'IntlProvider';

IntlProvider.childContextTypes = {
    intl: _types.intlShape.isRequired
};

IntlProvider.propTypes = _extends({}, _types.intlPropTypes, {
    children: _react.PropTypes.element.isRequired,
    initialNow: _react.PropTypes.any
});

IntlProvider.defaultProps = {
    formats: {},
    messages: {},

    defaultLocale: 'en',
    defaultFormats: {}
};
module.exports = exports['default'];